@@ -280,7 +280,7 @@ add_file_blame(const char *last_file,
 {
   if (!last_file)
     {
-      assert(chain->blame == NULL);
+      SVN_ERR_ASSERT(chain->blame == NULL);
       chain->blame = blame_create(chain, rev, 0);
     }
   else
@@ -425,6 +425,7 @@ file_rev_handler(void *baton, const char *path, svn_revnum_t revnum,
         = svn_wc_notify_state_inapplicable;
       notify->lock_state = svn_wc_notify_lock_state_inapplicable;
       notify->revision = revnum;
+      notify->rev_props = rev_props;
       frb->ctx->notify_func2(frb->ctx->notify_baton2, notify, pool);
     }
 
@@ -452,19 +453,19 @@ file_rev_handler(void *baton, const char *path, svn_revnum_t revnum,
   else
     /* Means empty stream below. */
     delta_baton->source_file = NULL;
-  last_stream = svn_stream_from_aprfile(delta_baton->source_file, pool);
+  last_stream = svn_stream_from_aprfile2(delta_baton->source_file, TRUE, pool);
 
   if (frb->include_merged_revisions && !frb->merged_revision)
     filepool = frb->filepool;
   else
     filepool = frb->currpool;
 
-  SVN_ERR(svn_io_open_unique_file2(&delta_baton->file,
+  SVN_ERR(svn_io_open_unique_file3(&delta_baton->file,
                                    &delta_baton->filename,
-                                   frb->tmp_path,
-                                   ".tmp", svn_io_file_del_on_pool_cleanup,
-                                   filepool));
-  cur_stream = svn_stream_from_aprfile(delta_baton->file, frb->currpool);
+                                   NULL,
+                                   svn_io_file_del_on_pool_cleanup,
+                                   filepool, filepool));
+  cur_stream = svn_stream_from_aprfile2(delta_baton->file, TRUE, frb->currpool);
 
   /* Get window handler for applying delta. */
   svn_txdelta_apply(last_stream, cur_stream, NULL, NULL,
@@ -483,7 +484,7 @@ file_rev_handler(void *baton, const char *path, svn_revnum_t revnum,
   if (revnum < frb->start_rev)
     {
       /* We shouldn't get more than one revision before start. */
-      assert(frb->last_filename == NULL);
+      SVN_ERR_ASSERT(frb->last_filename == NULL);
 
       /* The file existed before start_rev; generate no blame info for
          lines from this revision (or before). */
@@ -494,7 +495,7 @@ file_rev_handler(void *baton, const char *path, svn_revnum_t revnum,
   else
     {
       svn_string_t *str;
-      assert(revnum <= frb->end_rev);
+      SVN_ERR_ASSERT(revnum <= frb->end_rev);
 
       /* Set values from revision props. */
       frb->rev->revision = revnum;
@@ -605,8 +606,8 @@ svn_client_blame4(const char *target,
   const char *url;
   svn_revnum_t start_revnum, end_revnum;
   struct blame *walk, *walk_merged = NULL;
-  apr_file_t *file;
   apr_pool_t *iterpool;
+  svn_stream_t *last_stream;
   svn_stream_t *stream;
 
   if (start->kind == svn_opt_revision_unspecified
@@ -681,15 +682,15 @@ svn_client_blame4(const char *target,
   /* Report the blame to the caller. */
 
   /* The callback has to have been called at least once. */
-  assert(frb.last_filename != NULL);
+  SVN_ERR_ASSERT(frb.last_filename != NULL);
 
   /* Create a pool for the iteration below. */
   iterpool = svn_pool_create(pool);
 
   /* Open the last file and get a stream. */
-  SVN_ERR(svn_io_file_open(&file, frb.last_filename, APR_READ | APR_BUFFERED,
-                           APR_OS_DEFAULT, pool));
-  stream = svn_subst_stream_translated(svn_stream_from_aprfile(file, pool),
+  SVN_ERR(svn_stream_open_readonly(&last_stream, frb.last_filename,
+                                   pool, pool));
+  stream = svn_subst_stream_translated(last_stream,
                                        "\n", TRUE, NULL, FALSE, pool);
 
   /* Perform optional merged chain normalization. */
@@ -755,9 +756,6 @@ svn_client_blame4(const char *target,
 
   SVN_ERR(svn_stream_close(stream));
 
-  /* We don't need the temp file any more. */
-  SVN_ERR(svn_io_file_close(file, pool));
-
   svn_pool_destroy(frb.lastpool);
   svn_pool_destroy(frb.currpool);
   if (include_merged_revisions)
@@ -769,152 +767,3 @@ svn_client_blame4(const char *target,
 
   return SVN_NO_ERROR;
 }
-
-/* Baton for use with wrap_blame_receiver */
-struct blame_receiver_wrapper_baton {
-  void *baton;
-  svn_client_blame_receiver_t receiver;
-};
-
-/* This implements svn_client_blame_receiver2_t */
-static svn_error_t *
-blame_wrapper_receiver(void *baton,
-                       apr_int64_t line_no,
-                       svn_revnum_t revision,
-                       const char *author,
-                       const char *date,
-                       svn_revnum_t merged_revision,
-                       const char *merged_author,
-                       const char *merged_date,
-                       const char *merged_path,
-                       const char *line,
-                       apr_pool_t *pool)
-{
-  struct blame_receiver_wrapper_baton *brwb = baton;
-
-  if (brwb->receiver)
-    return brwb->receiver(brwb->baton,
-                          line_no, revision, author, date, line, pool);
-
-  return SVN_NO_ERROR;
-}
-
-static void
-wrap_blame_receiver(svn_client_blame_receiver2_t *receiver2,
-                    void **receiver2_baton,
-                    svn_client_blame_receiver_t receiver,
-                    void *receiver_baton,
-                    apr_pool_t *pool)
-{
-  struct blame_receiver_wrapper_baton *brwb = apr_palloc(pool, sizeof(*brwb));
-
-  /* Set the user provided old format callback in the baton. */
-  brwb->baton = receiver_baton;
-  brwb->receiver = receiver;
-
-  *receiver2_baton = brwb;
-  *receiver2 = blame_wrapper_receiver;
-}
-
-svn_error_t *
-svn_client_blame3(const char *target,
-                  const svn_opt_revision_t *peg_revision,
-                  const svn_opt_revision_t *start,
-                  const svn_opt_revision_t *end,
-                  const svn_diff_file_options_t *diff_options,
-                  svn_boolean_t ignore_mime_type,
-                  svn_client_blame_receiver_t receiver,
-                  void *receiver_baton,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
-{
-  svn_client_blame_receiver2_t receiver2;
-  void *receiver2_baton;
-
-  wrap_blame_receiver(&receiver2, &receiver2_baton, receiver, receiver_baton,
-                      pool);
-
-  return svn_client_blame4(target, peg_revision, start, end, diff_options,
-                           ignore_mime_type, FALSE, receiver2, receiver2_baton,
-                           ctx, pool);
-}
-
-/* svn_client_blame3 guarantees 'no EOL chars' as part of the receiver
-   LINE argument.  Older versions depend on the fact that if a CR is
-   required, that CR is already part of the LINE data.
-
-   Because of this difference, we need to trap old receivers and append
-   a CR to LINE before passing it on to the actual receiver on platforms
-   which want CRLF line termination.
-
-*/
-
-struct wrapped_receiver_baton_s
-{
-  svn_client_blame_receiver_t orig_receiver;
-  void *orig_baton;
-};
-
-static svn_error_t *
-wrapped_receiver(void *baton,
-                 apr_int64_t line_no,
-                 svn_revnum_t revision,
-                 const char *author,
-                 const char *date,
-                 const char *line,
-                 apr_pool_t *pool)
-{
-  struct wrapped_receiver_baton_s *b = baton;
-  svn_stringbuf_t *expanded_line = svn_stringbuf_create(line, pool);
-
-  svn_stringbuf_appendbytes(expanded_line, "\r", 1);
-
-  return b->orig_receiver(b->orig_baton, line_no, revision, author,
-                          date, expanded_line->data, pool);
-}
-
-static void
-wrap_pre_blame3_receiver(svn_client_blame_receiver_t *receiver,
-                   void **receiver_baton,
-                   apr_pool_t *pool)
-{
-  if (strlen(APR_EOL_STR) > 1)
-    {
-      struct wrapped_receiver_baton_s *b = apr_palloc(pool,sizeof(*b));
-
-      b->orig_receiver = *receiver;
-      b->orig_baton = *receiver_baton;
-
-      *receiver_baton = b;
-      *receiver = wrapped_receiver;
-    }
-}
-
-svn_error_t *
-svn_client_blame2(const char *target,
-                  const svn_opt_revision_t *peg_revision,
-                  const svn_opt_revision_t *start,
-                  const svn_opt_revision_t *end,
-                  svn_client_blame_receiver_t receiver,
-                  void *receiver_baton,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
-{
-  wrap_pre_blame3_receiver(&receiver, &receiver_baton, pool);
-  return svn_client_blame3(target, peg_revision, start, end,
-                           svn_diff_file_options_create(pool), FALSE,
-                           receiver, receiver_baton, ctx, pool);
-}
-svn_error_t *
-svn_client_blame(const char *target,
-                 const svn_opt_revision_t *start,
-                 const svn_opt_revision_t *end,
-                 svn_client_blame_receiver_t receiver,
-                 void *receiver_baton,
-                 svn_client_ctx_t *ctx,
-                 apr_pool_t *pool)
-{
-  wrap_pre_blame3_receiver(&receiver, &receiver_baton, pool);
-  return svn_client_blame2(target, end, start, end,
-                           receiver, receiver_baton, ctx, pool);
-}
