@@ -2,7 +2,7 @@
  * commit.c :  entry point for commit RA functions for ra_serf
  *
  * ====================================================================
- * Copyright (c) 2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2006, 2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,8 +16,6 @@
  * ====================================================================
  */
 
-
-
 #include <apr_uri.h>
 
 #include <expat.h>
@@ -28,16 +26,18 @@
 #include "svn_ra.h"
 #include "svn_dav.h"
 #include "svn_xml.h"
-#include "../libsvn_ra/ra_loader.h"
 #include "svn_config.h"
 #include "svn_delta.h"
 #include "svn_base64.h"
 #include "svn_version.h"
 #include "svn_path.h"
+#include "svn_props.h"
+
 #include "svn_private_config.h"
 #include "private/svn_dep_compat.h"
 
 #include "ra_serf.h"
+#include "../libsvn_ra/ra_loader.h"
 
 
 /* Structure associated with a CHECKOUT request. */
@@ -214,42 +214,41 @@ static svn_error_t *
 return_response_err(svn_ra_serf__handler_t *handler,
                     svn_ra_serf__simple_request_context_t *ctx)
 {
-  SVN_ERR(ctx->server_error.error);
-
-  return svn_error_createf(SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
-                           "%s of '%s': %d %s",
-                           handler->method, handler->path,
-                           ctx->status, ctx->reason);
+  /* Try to return one of the standard errors for 301, 404, etc.,
+     then look for an error embedded in the response.  */
+  return svn_error_compose_create(
+    svn_ra_serf__error_on_status(ctx->status, handler->path),
+    svn_error_compose_create(
+      ctx->server_error.error,
+      svn_error_createf(SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
+                        "%s of '%s': %d %s",
+                        handler->method, handler->path,
+                        ctx->status, ctx->reason)));
 }
 
-#define CHECKOUT_HEADER "<?xml version=\"1.0\" encoding=\"utf-8\"?><D:checkout xmlns:D=\"DAV:\"><D:activity-set><D:href>"
-
-#define CHECKOUT_TRAILER "</D:href></D:activity-set></D:checkout>"
-
 static serf_bucket_t *
 create_checkout_body(void *baton,
                      serf_bucket_alloc_t *alloc,
                      apr_pool_t *pool)
 {
   checkout_context_t *ctx = baton;
-  serf_bucket_t *body_bkt, *tmp_bkt;
+  serf_bucket_t *body_bkt;
 
   body_bkt = serf_bucket_aggregate_create(alloc);
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(CHECKOUT_HEADER,
-                                          sizeof(CHECKOUT_HEADER) - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_xml_header_buckets(body_bkt, alloc);
+  svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:checkout",
+                                    "xmlns:D", "DAV:",
+                                    NULL);
+  svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:activity-set", NULL);
+  svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:href", NULL);
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(ctx->activity_url,
-                                          ctx->activity_url_len,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_cdata_len_buckets(body_bkt, alloc,
+                                     ctx->activity_url, ctx->activity_url_len);
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(CHECKOUT_TRAILER,
-                                          sizeof(CHECKOUT_TRAILER) - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:href");
+  svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:activity-set");
+  svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:checkout");
 
   return body_bkt;
 }
@@ -413,15 +412,7 @@ checkout_dir(dir_context_t *dir)
 
   if (checkout_ctx->progress.status != 201)
     {
-      SVN_ERR(svn_ra_serf__error_on_status(checkout_ctx->progress.status, 
-                                           dir->name));
-
-      return svn_error_createf(SVN_ERR_FS_CONFLICT,
-                    return_response_err(handler,
-                                        &checkout_ctx->progress),
-                    _("Directory '%s' is out of date; try updating"),
-                    svn_path_local_style(relative_dir_path(dir, dir->pool),
-                                         dir->pool));
+      return return_response_err(handler, &checkout_ctx->progress);
     }
 
   return SVN_NO_ERROR;
@@ -522,7 +513,7 @@ checkout_file(file_context_t *file)
         {
           dir = dir->parent_dir;
         }
-          
+
 
       /* Is our parent a copy?  If so, we're already implicitly checked out. */
       if (dir)
@@ -590,15 +581,7 @@ checkout_file(file_context_t *file)
 
   if (file->checkout->progress.status != 201)
     {
-      SVN_ERR(svn_ra_serf__error_on_status(file->checkout->progress.status,
-                                           file->name));
-
-      return svn_error_createf(SVN_ERR_FS_CONFLICT,
-                    return_response_err(handler,
-                                        &file->checkout->progress),
-                    _("File '%s' is out of date; try updating"),
-                    svn_path_local_style(relative_file_path(file, file->pool),
-                                         file->pool));
+      return return_response_err(handler, &file->checkout->progress);
     }
 
   return SVN_NO_ERROR;
@@ -612,7 +595,6 @@ proppatch_walker(void *baton,
                  apr_pool_t *pool)
 {
   serf_bucket_t *body_bkt = baton;
-  serf_bucket_t *tmp_bkt;
   serf_bucket_alloc_t *alloc;
   svn_boolean_t binary_prop;
   char *prop_name;
@@ -636,55 +618,24 @@ proppatch_walker(void *baton,
 
   alloc = body_bkt->allocator;
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("<",
-                                          sizeof("<") - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(prop_name, name_len, alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, prop_name,
+                                    "V:encoding", binary_prop ? "base64" : NULL,
+                                    NULL);
 
   if (binary_prop == TRUE)
     {
-      tmp_bkt =
-          SERF_BUCKET_SIMPLE_STRING_LEN(" V:encoding=\"base64\"",
-                                        sizeof(" V:encoding=\"base64\"") - 1,
-                                        alloc);
+      serf_bucket_t *tmp_bkt;
+      val = svn_base64_encode_string2(val, TRUE, pool);
+      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(val->data, val->len, alloc);
       serf_bucket_aggregate_append(body_bkt, tmp_bkt);
     }
-
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(">",
-                                          sizeof(">") - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-  if (binary_prop == TRUE)
-    {
-      val = svn_base64_encode_string(val, pool);
-    }
   else
     {
-      svn_stringbuf_t *prop_buf = svn_stringbuf_create("", pool);
-      svn_xml_escape_cdata_string(&prop_buf, val, pool);
-      val = svn_string_create_from_buf(prop_buf, pool);
+      svn_ra_serf__add_cdata_len_buckets(body_bkt, alloc, val->data, val->len);
     }
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(val->data, val->len, alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("</",
-                                          sizeof("</") - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(prop_name, name_len, alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(">",
-                                          sizeof(">") - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
 
+  svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, prop_name);
   return SVN_NO_ERROR;
 }
 
@@ -715,87 +666,51 @@ setup_proppatch_headers(serf_bucket_t *headers,
   return APR_SUCCESS;
 }
 
-#define PROPPATCH_HEADER "<?xml version=\"1.0\" encoding=\"utf-8\"?>"\
-                         "<D:propertyupdate xmlns:D=\"DAV:\" "\
-                         "xmlns:V=\"" SVN_DAV_PROP_NS_DAV "\" "\
-                         "xmlns:C=\"" SVN_DAV_PROP_NS_CUSTOM "\" "\
-                         "xmlns:S=\"" SVN_DAV_PROP_NS_SVN "\">"
-
-#define PROPPATCH_TRAILER "</D:propertyupdate>"
-
 static serf_bucket_t *
 create_proppatch_body(void *baton,
                       serf_bucket_alloc_t *alloc,
                       apr_pool_t *pool)
 {
   proppatch_context_t *ctx = baton;
-  serf_bucket_t *body_bkt, *tmp_bkt;
+  serf_bucket_t *body_bkt;
 
   body_bkt = serf_bucket_aggregate_create(alloc);
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(PROPPATCH_HEADER,
-                                          sizeof(PROPPATCH_HEADER) - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_xml_header_buckets(body_bkt, alloc);
+  svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:propertyupdate",
+                                    "xmlns:D", "DAV:",
+                                    "xmlns:V", SVN_DAV_PROP_NS_DAV,
+                                    "xmlns:C", SVN_DAV_PROP_NS_CUSTOM,
+                                    "xmlns:S", SVN_DAV_PROP_NS_SVN,
+                                    NULL);
 
   if (apr_hash_count(ctx->changed_props) > 0)
     {
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("<D:set>",
-                                              sizeof("<D:set>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("<D:prop>",
-                                              sizeof("<D:prop>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+      svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:set", NULL);
+      svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:prop", NULL);
 
       svn_ra_serf__walk_all_props(ctx->changed_props, ctx->path,
                                   SVN_INVALID_REVNUM,
                                   proppatch_walker, body_bkt, pool);
 
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("</D:prop>",
-                                              sizeof("</D:prop>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("</D:set>",
-                                              sizeof("</D:set>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+      svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:prop");
+      svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:set");
     }
 
   if (apr_hash_count(ctx->removed_props) > 0)
     {
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("<D:remove>",
-                                              sizeof("<D:remove>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("<D:prop>",
-                                              sizeof("<D:prop>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+      svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:remove", NULL);
+      svn_ra_serf__add_open_tag_buckets(body_bkt, alloc, "D:prop", NULL);
 
       svn_ra_serf__walk_all_props(ctx->removed_props, ctx->path,
                                   SVN_INVALID_REVNUM,
                                   proppatch_walker, body_bkt, pool);
 
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("</D:prop>",
-                                              sizeof("</D:prop>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
-
-      tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN("</D:remove>",
-                                              sizeof("</D:remove>") - 1,
-                                              alloc);
-      serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+      svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:prop");
+      svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:remove");
     }
 
-  tmp_bkt = SERF_BUCKET_SIMPLE_STRING_LEN(PROPPATCH_TRAILER,
-                                          sizeof(PROPPATCH_TRAILER) - 1,
-                                          alloc);
-  serf_bucket_aggregate_append(body_bkt, tmp_bkt);
+  svn_ra_serf__add_close_tag_buckets(body_bkt, alloc, "D:propertyupdate");
 
   return body_bkt;
 }
@@ -831,10 +746,9 @@ proppatch_resource(proppatch_context_t *proppatch,
   if (proppatch->progress.status != 207 ||
       proppatch->progress.server_error.error)
     {
-      svn_error_t *err;
-      err = return_response_err(handler, &proppatch->progress);
-      return svn_error_create(SVN_ERR_RA_DAV_PROPPATCH_FAILED, err,
-             _("At least one property change failed; repository is unchanged"));
+      return svn_error_create(SVN_ERR_RA_DAV_PROPPATCH_FAILED,
+        return_response_err(handler, &proppatch->progress),
+        _("At least one property change failed; repository is unchanged"));
     }
 
   return SVN_NO_ERROR;
@@ -1006,21 +920,17 @@ setup_delete_headers(serf_bucket_t *headers,
   return APR_SUCCESS;
 }
 
-#define XML_HEADER "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
-
 static serf_bucket_t *
 create_delete_body(void *baton,
                    serf_bucket_alloc_t *alloc,
                    apr_pool_t *pool)
 {
   delete_context_t *ctx = baton;
-  serf_bucket_t *body, *tmp;
+  serf_bucket_t *body;
 
   body = serf_bucket_aggregate_create(alloc);
 
-  tmp = SERF_BUCKET_SIMPLE_STRING_LEN(XML_HEADER, sizeof(XML_HEADER) - 1,
-                                      alloc);
-  serf_bucket_aggregate_append(body, tmp);
+  svn_ra_serf__add_xml_header_buckets(body, alloc);
 
   svn_ra_serf__merge_lock_token_list(ctx->lock_token_hash, ctx->path,
                                      body, alloc, pool);
@@ -1277,11 +1187,8 @@ delete_entry(const char *path,
       return err;
     }
 
-  /* 204 No Content: item successfully deleted
-     404 Not found:  ignored, the item might have been deleted in this
-                     transaction. */
-  if (delete_ctx->progress.status != 204 &&
-      delete_ctx->progress.status != 404)
+  /* 204 No Content: item successfully deleted */
+  if (delete_ctx->progress.status != 204)
     {
       return return_response_err(handler, &delete_ctx->progress);
     }
@@ -1524,7 +1431,7 @@ absent_directory(const char *path,
   dir_context_t *ctx = parent_baton;
 #endif
 
-  abort();
+  SVN_ERR_MALFUNCTION();
 }
 
 static svn_error_t *
@@ -1561,7 +1468,7 @@ add_file(const char *path,
   new_file->changed_props = apr_hash_make(new_file->pool);
   new_file->removed_props = apr_hash_make(new_file->pool);
 
-  /* Ensure that the file doesn't exist by doing a HEAD on the 
+  /* Ensure that the file doesn't exist by doing a HEAD on the
    * resource, but only if we haven't deleted it in this commit
    * already - directly, or indirectly through its parent directories -
    * or if the parent directory was also added (without history)
@@ -1680,14 +1587,15 @@ apply_textdelta(void *file_baton,
    */
   wc_callbacks = ctx->commit->session->wc_callbacks;
   wc_callback_baton = ctx->commit->session->wc_callback_baton;
-  SVN_ERR(wc_callbacks->open_tmp_file(&ctx->svndiff,
-                                      wc_callback_baton,
-                                      ctx->pool));
+
+  SVN_ERR(svn_io_open_unique_file3(&ctx->svndiff, NULL, NULL,
+                                   svn_io_file_del_on_pool_cleanup,
+                                   ctx->pool, ctx->pool));
 
   ctx->stream = svn_stream_create(ctx, pool);
   svn_stream_set_write(ctx->stream, svndiff_stream_write);
 
-  svn_txdelta_to_svndiff(ctx->stream, pool, handler, handler_baton);
+  svn_txdelta_to_svndiff2(handler, handler_baton, ctx->stream, 0, pool);
 
   ctx->base_checksum = base_checksum;
 
@@ -1887,7 +1795,7 @@ absent_file(const char *path,
   dir_context_t *ctx = parent_baton;
 #endif
 
-  abort();
+  SVN_ERR_MALFUNCTION();
 }
 
 static svn_error_t *
@@ -1916,7 +1824,7 @@ close_edit(void *edit_baton,
 
   if (svn_ra_serf__merge_get_status(merge_ctx) != 200)
     {
-      abort();
+      SVN_ERR_MALFUNCTION();
     }
 
   /* Inform the WC that we did a commit.  */
@@ -1940,10 +1848,7 @@ close_edit(void *edit_baton,
   SVN_ERR(svn_ra_serf__context_run_wait(&delete_ctx->done, ctx->session,
                                         pool));
 
-  if (delete_ctx->status != 204)
-    {
-      abort();
-    }
+  SVN_ERR_ASSERT(delete_ctx->status == 204);
 
   return SVN_NO_ERROR;
 }
@@ -1985,7 +1890,7 @@ abort_edit(void *edit_baton,
       delete_ctx->status != 404
       )
     {
-      abort();
+      SVN_ERR_MALFUNCTION();
     }
 
   return SVN_NO_ERROR;
