@@ -2,7 +2,7 @@
  * reporter.c : `reporter' vtable routines for updates.
  *
  * ====================================================================
- * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006, 2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -23,7 +23,6 @@
 #include "svn_fs.h"
 #include "svn_repos.h"
 #include "svn_pools.h"
-#include "svn_md5.h"
 #include "svn_props.h"
 #include "repos.h"
 #include "svn_private_config.h"
@@ -231,7 +230,7 @@ read_depth(svn_depth_t *depth, apr_file_t *temp, const char *path,
       return svn_error_createf(SVN_ERR_REPOS_BAD_REVISION_REPORT, NULL,
                                _("Invalid depth (%c) for path '%s'"), c, path);
     }
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -526,7 +525,7 @@ delta_files(report_baton_t *b, void *file_baton, svn_revnum_t s_rev,
   svn_boolean_t changed;
   svn_fs_root_t *s_root = NULL;
   svn_txdelta_stream_t *dstream = NULL;
-  unsigned char s_digest[APR_MD5_DIGESTSIZE];
+  svn_checksum_t *s_checksum;
   const char *s_hex_digest = NULL;
   svn_txdelta_window_handler_t dhandler;
   void *dbaton;
@@ -554,8 +553,9 @@ delta_files(report_baton_t *b, void *file_baton, svn_revnum_t s_rev,
       if (!changed)
         return SVN_NO_ERROR;
 
-      SVN_ERR(svn_fs_file_md5_checksum(s_digest, s_root, s_path, pool));
-      s_hex_digest = svn_md5_digest_to_cstring(s_digest, pool);
+      SVN_ERR(svn_fs_file_checksum(&s_checksum, svn_checksum_md5, s_root,
+                                   s_path, TRUE, pool));
+      s_hex_digest = svn_checksum_to_cstring(s_checksum, pool);
     }
 
   /* Send the delta stream if desired, or just a NULL window if not. */
@@ -701,10 +701,9 @@ add_file_smartly(report_baton_t *b,
         }
     }
 
-  SVN_ERR(b->editor->add_file(path, parent_baton,
-                              *copyfrom_path, *copyfrom_rev,
-                              pool, new_file_baton));
-  return SVN_NO_ERROR;
+  return b->editor->add_file(path, parent_baton,
+                             *copyfrom_path, *copyfrom_rev,
+                             pool, new_file_baton);
 }
 
 
@@ -747,7 +746,7 @@ update_entry(report_baton_t *b, svn_revnum_t s_rev, const char *s_path,
   svn_fs_root_t *s_root;
   svn_boolean_t allowed, related;
   void *new_baton;
-  unsigned char digest[APR_MD5_DIGESTSIZE];
+  svn_checksum_t *checksum;
   const char *hex_digest;
   int distance;
 
@@ -866,8 +865,9 @@ update_entry(report_baton_t *b, svn_revnum_t s_rev, const char *s_path,
                                 t_path, info ? info->lock_token : NULL, pool));
         }
 
-      SVN_ERR(svn_fs_file_md5_checksum(digest, b->t_root, t_path, pool));
-      hex_digest = svn_md5_digest_to_cstring(digest, pool);
+      SVN_ERR(svn_fs_file_checksum(&checksum, svn_checksum_md5, b->t_root,
+                                   t_path, TRUE, pool));
+      hex_digest = svn_checksum_to_cstring(checksum, pool);
       return b->editor->close_file(new_baton, hex_digest, pool);
     }
 }
@@ -976,7 +976,7 @@ delta_dirs(report_baton_t *b, svn_revnum_t s_rev, const char *s_path,
 
           /* Invalid revnum means we should delete, unless this is
              just an excluded subpath. */
-          if (info 
+          if (info
               && !SVN_IS_VALID_REVNUM(info->rev)
               && info->depth != svn_depth_exclude)
             {
@@ -1017,7 +1017,10 @@ delta_dirs(report_baton_t *b, svn_revnum_t s_rev, const char *s_path,
 
           /* Don't revisit this name in the target or source entries. */
           apr_hash_set(t_entries, name, APR_HASH_KEY_STRING, NULL);
-          if (s_entries)
+          if (s_entries
+              /* Keep the entry for later process if it is reported as
+                 excluded and got deleted in repos. */
+              && (! info || info->depth != svn_depth_exclude || t_entry))
             apr_hash_set(s_entries, name, APR_HASH_KEY_STRING, NULL);
 
           /* pathinfo entries live in their own subpools due to lookahead,
@@ -1156,8 +1159,9 @@ drive(report_baton_t *b, svn_revnum_t s_rev, path_info_t *info,
 
   /* Check if the target path exists first.  */
   if (!*b->s_operand && !(t_entry))
-    return svn_error_create(SVN_ERR_FS_PATH_SYNTAX, NULL,
-                            _("Target path does not exist"));
+    return svn_error_createf(SVN_ERR_FS_PATH_SYNTAX, NULL,
+                             _("Target path '%s' does not exist"),
+                             b->t_path);
 
   /* If the anchor is the operand, the source and target must be dirs.
      Check this before opening the root to avoid modifying the wc. */
@@ -1179,9 +1183,7 @@ drive(report_baton_t *b, svn_revnum_t s_rev, path_info_t *info,
                          t_entry, root_baton, b->s_operand, info,
                          info->depth, b->requested_depth, pool));
 
-  SVN_ERR(b->editor->close_directory(root_baton, pool));
-  SVN_ERR(b->editor->close_edit(b->edit_baton, pool));
-  return SVN_NO_ERROR;
+  return b->editor->close_directory(root_baton, pool);
 }
 
 /* Initialize the baton fields for editor-driving, and drive the editor. */
@@ -1239,7 +1241,13 @@ finish_report(report_baton_t *b, apr_pool_t *pool)
   for (i = 0; i < NUM_CACHED_SOURCE_ROOTS; i++)
     b->s_roots[i] = NULL;
 
-  return drive(b, s_rev, info, pool);
+  {
+    svn_error_t *err = drive(b, s_rev, info, pool);
+    if (err == SVN_NO_ERROR)
+      return b->editor->close_edit(b->edit_baton, pool);
+    svn_error_clear(b->editor->abort_edit(b->edit_baton, pool));
+    return err;
+  }
 }
 
 /* --- COLLECTING THE REPORT INFORMATION --- */
@@ -1295,22 +1303,6 @@ svn_repos_set_path3(void *baton, const char *path, svn_revnum_t rev,
                          lock_token, pool);
 }
 
-svn_error_t *
-svn_repos_set_path2(void *baton, const char *path, svn_revnum_t rev,
-                    svn_boolean_t start_empty, const char *lock_token,
-                    apr_pool_t *pool)
-{
-  return svn_repos_set_path3(baton, path, rev, svn_depth_infinity,
-                             start_empty, lock_token, pool);
-}
-
-svn_error_t *
-svn_repos_set_path(void *baton, const char *path, svn_revnum_t rev,
-                   svn_boolean_t start_empty, apr_pool_t *pool)
-{
-  return svn_repos_set_path2(baton, path, rev, start_empty, NULL, pool);
-}
-
 svn_error_t *
 svn_repos_link_path3(void *baton, const char *path, const char *link_path,
                      svn_revnum_t rev, svn_depth_t depth,
@@ -1325,24 +1317,6 @@ svn_repos_link_path3(void *baton, const char *path, const char *link_path,
                          start_empty, lock_token, pool);
 }
 
-svn_error_t *
-svn_repos_link_path2(void *baton, const char *path, const char *link_path,
-                     svn_revnum_t rev, svn_boolean_t start_empty,
-                     const char *lock_token, apr_pool_t *pool)
-{
-  return svn_repos_link_path3(baton, path, link_path, rev, svn_depth_infinity,
-                              start_empty, lock_token, pool);
-}
-
-svn_error_t *
-svn_repos_link_path(void *baton, const char *path, const char *link_path,
-                    svn_revnum_t rev, svn_boolean_t start_empty,
-                    apr_pool_t *pool)
-{
-  return svn_repos_link_path2(baton, path, link_path, rev, start_empty,
-                              NULL, pool);
-}
-
 svn_error_t *
 svn_repos_delete_path(void *baton, const char *path, apr_pool_t *pool)
 {
@@ -1394,7 +1368,6 @@ svn_repos_begin_report2(void **report_baton,
                         apr_pool_t *pool)
 {
   report_baton_t *b;
-  const char *tempdir;
 
   if (depth == svn_depth_exclude)
     return svn_error_create(SVN_ERR_REPOS_BAD_ARGS, NULL,
@@ -1419,47 +1392,11 @@ svn_repos_begin_report2(void **report_baton,
   b->authz_read_func = authz_read_func;
   b->authz_read_baton = authz_read_baton;
 
-  SVN_ERR(svn_io_temp_dir(&tempdir, pool));
-  SVN_ERR(svn_io_open_unique_file2(&b->tempfile, NULL,
-                                   apr_psprintf(pool, "%s/report", tempdir),
-                                   ".tmp", svn_io_file_del_on_close, pool));
+  SVN_ERR(svn_io_open_unique_file3(&b->tempfile, NULL, NULL,
+                                   svn_io_file_del_on_pool_cleanup,
+                                   pool, pool));
 
   /* Hand reporter back to client. */
   *report_baton = b;
   return SVN_NO_ERROR;
 }
-
-
-svn_error_t *
-svn_repos_begin_report(void **report_baton,
-                       svn_revnum_t revnum,
-                       const char *username,
-                       svn_repos_t *repos,
-                       const char *fs_base,
-                       const char *s_operand,
-                       const char *switch_path,
-                       svn_boolean_t text_deltas,
-                       svn_boolean_t recurse,
-                       svn_boolean_t ignore_ancestry,
-                       const svn_delta_editor_t *editor,
-                       void *edit_baton,
-                       svn_repos_authz_func_t authz_read_func,
-                       void *authz_read_baton,
-                       apr_pool_t *pool)
-{
-  return svn_repos_begin_report2(report_baton,
-                                 revnum,
-                                 repos,
-                                 fs_base,
-                                 s_operand,
-                                 switch_path,
-                                 text_deltas,
-                                 SVN_DEPTH_INFINITY_OR_FILES(recurse),
-                                 ignore_ancestry,
-                                 FALSE, /* don't send copyfrom args */
-                                 editor,
-                                 edit_baton,
-                                 authz_read_func,
-                                 authz_read_baton,
-                                 pool);
-}
