@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2006 CollabNet.  All rights reserved.
+# Copyright (c) 2006, 2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,8 +21,8 @@ import os
 
 def main():
     if len(sys.argv) not in [2, 5]:
-        print "usage: svneditor.py file"
-        print "       svneditor.py base theirs mine merged"
+        print("usage: svneditor.py file")
+        print("       svneditor.py base theirs mine merged")
         sys.exit(1)
 
     filename = sys.argv[-1]
